from frozendict import frozendict

from bgpy.tests.engine_tests.graphs import graph_014
from bgpy.tests.engine_tests.utils import EngineTestConfig

from bgpy.simulation_engine import BGPSimpleAS, ROVSimpleAS
from bgpy.enums import ASNs
from bgpy.simulation_framework import ScenarioConfig, SubprefixHijack

from rovpp import ROVPPAnn


config_187 = EngineTestConfig(
    name="187",
    desc="Subprefix Hijack Attack with ROV",
    scenario_config=ScenarioConfig(
        ScenarioCls=SubprefixHijack,
        BaseASCls=BGPSimpleAS,
        AdoptASCls=ROVSimpleAS,
        AnnCls=ROVPPAnn,
        override_attacker_asns=frozenset({ASNs.ATTACKER.value}),
        override_victim_asns=frozenset({ASNs.VICTIM.value}),
        override_non_default_asn_cls_dict=frozendict({6: ROVSimpleAS}),
    ),
    graph=graph_014,
    propagation_rounds=1,
)



import pytest

from ..__main__ import main


@pytest.mark.slow
@pytest.mark.skip(reason="Takes 10m, not worth it")
class TestMain:
    def test_main(self):
        """Runs through the graphs quickly, simple sanity check"""

        main(quick=True)



from frozendict import frozendict

from bgpy.tests.engine_tests.graphs import graph_016
from bgpy.tests.engine_tests.utils import EngineTestConfig

from bgpy.simulation_engine import BGPSimpleAS
from bgpy.enums import ASNs
from bgpy.simulation_framework import ScenarioConfig, SuperprefixPrefixHijack

from rovpp import ROVPPAnn, ROVPPV2SimpleAS


config_063 = EngineTestConfig(
    name="063",
    desc=(
        "This is running as single adopting AS (ROV++ v2 AS 12). \n"
        "This is 22 AS topology "
        "with a clique at the top with 1,2,3, and 4.\nIts about 4 levels "
        "tall.\nThe attacker under the same provider as the legitmated "
        "origin\n(i.e. they’re both on the edge next to each other)."
    ),
    scenario_config=ScenarioConfig(
        ScenarioCls=SuperprefixPrefixHijack,
        BaseASCls=BGPSimpleAS,
        AdoptASCls=ROVPPV2SimpleAS,
        override_attacker_asns=frozenset({ASNs.ATTACKER.value}),
        override_victim_asns=frozenset({ASNs.VICTIM.value}),
        override_non_default_asn_cls_dict=frozendict({12: ROVPPV2SimpleAS}),
        AnnCls=ROVPPAnn,
    ),
    graph=graph_016,
)



from frozendict import frozendict

from bgpy.tests.engine_tests.graphs import graph_051
from bgpy.tests.engine_tests.utils import EngineTestConfig

from bgpy.simulation_engine import BGPSimpleAS
from bgpy.enums import ASNs
from bgpy.simulation_framework import ScenarioConfig, NonRoutedPrefixHijack

from rovpp import ROVPPAnn, ROVPPV2aSimpleAS


config_090 = EngineTestConfig(
    name="090",
    desc="NonRouted Prefix Hijack with v2a",
    scenario_config=ScenarioConfig(
        ScenarioCls=NonRoutedPrefixHijack,
        BaseASCls=BGPSimpleAS,
        AdoptASCls=ROVPPV2aSimpleAS,
        override_attacker_asns=frozenset({ASNs.ATTACKER.value}),
        override_victim_asns=frozenset({ASNs.VICTIM.value}),
        override_non_default_asn_cls_dict=frozendict({4: ROVPPV2aSimpleAS}),
        AnnCls=ROVPPAnn,
    ),
    graph=graph_051,
)



from frozendict import frozendict

from bgpy.tests.engine_tests.graphs import graph_005
from bgpy.tests.engine_tests.utils import EngineTestConfig

from bgpy.simulation_engine import BGPSimpleAS
from bgpy.enums import ASNs
from bgpy.simulation_framework import ScenarioConfig, SubprefixHijack

from rovpp import ROVPPV3AS, ROVPPAnn


config_045 = EngineTestConfig(
    name="045",
    desc="Subprefix Hijack to test ROV++ v3.",
    scenario_config=ScenarioConfig(
        ScenarioCls=SubprefixHijack,
        BaseASCls=BGPSimpleAS,
        AdoptASCls=ROVPPV3AS,
        override_attacker_asns=frozenset({ASNs.ATTACKER.value}),
        override_victim_asns=frozenset({ASNs.VICTIM.value}),
        override_non_default_asn_cls_dict=frozendict(
            {
                5: ROVPPV3AS,
                8: ROVPPV3AS,
                10: ROVPPV3AS,
                15: ROVPPV3AS,
                18: ROVPPV3AS,
            }
        ),
        AnnCls=ROVPPAnn,
    ),
    graph=graph_005,
)



from ..v2_base import ROVPPV2LiteSimpleAS


class ROVPPV2aLiteSimpleAS(ROVPPV2LiteSimpleAS):
    name = "ROV++V2a Lite Simple"

    def _policy_propagate(*args, **kwargs):
        """Do nothing. Send blackholes according to export policy"""

        return False



from ..v2_base import ROVPPV2LiteSimpleAS


class ROVPPV2ShortenLiteSimpleAS(ROVPPV2LiteSimpleAS):
    name = "ROV++V2 Shorten Lite Simple"

    def _copy_and_process(self, ann, recv_relationship, overwrite_default_kwargs=None):
        """Deep copies ann and modifies attrs"""

        if overwrite_default_kwargs:
            overwrite_default_kwargs["holes"] = self.temp_holes[ann]
        else:
            overwrite_default_kwargs = {"holes": self.temp_holes[ann]}
        if ann.invalid_by_roa and not ann.preventive:
            # tuple([ann.as_path[-1]])
            overwrite_default_kwargs["as_path"] = tuple([self.asn])
            overwrite_default_kwargs["blackhole"] = True

        return super(ROVPPV2LiteSimpleAS, self)._copy_and_process(
            ann, recv_relationship, overwrite_default_kwargs=overwrite_default_kwargs
        )



from frozendict import frozendict

from bgpy.tests.engine_tests.graphs import graph_006
from bgpy.tests.engine_tests.utils import EngineTestConfig

from bgpy.simulation_engine import BGPSimpleAS
from bgpy.enums import ASNs
from bgpy.simulation_framework import ScenarioConfig, NonRoutedSuperprefixPrefixHijack

from rovpp import ROVPPAnn
from rovpp import ROVPPV2SimpleAS


config_101 = EngineTestConfig(
    name="101",
    desc="Superprefix+prefix Attack on NonRouted Prefix with v2",
    scenario_config=ScenarioConfig(
        ScenarioCls=NonRoutedSuperprefixPrefixHijack,
        BaseASCls=BGPSimpleAS,
        AdoptASCls=ROVPPV2SimpleAS,
        override_attacker_asns=frozenset({ASNs.ATTACKER.value}),
        override_victim_asns=frozenset({ASNs.VICTIM.value}),
        override_non_default_asn_cls_dict=frozendict({2: ROVPPV2SimpleAS}),
        AnnCls=ROVPPAnn,
    ),
    graph=graph_006,
    propagation_rounds=1,
)



def _copy_and_process(self, ann, recv_relationship, overwrite_default_kwargs=None):
    """Deep copies ann and modifies attrs"""

    if overwrite_default_kwargs:
        overwrite_default_kwargs["holes"] = self.temp_holes[ann]
    else:
        overwrite_default_kwargs = {"holes": self.temp_holes[ann]}
    if ann.invalid_by_roa and not ann.preventive:
        # tuple([ann.as_path[-1]])
        overwrite_default_kwargs["as_path"] = tuple([self.asn])
        overwrite_default_kwargs["blackhole"] = True

    return super()._copy_and_process(
        ann, recv_relationship, overwrite_default_kwargs=overwrite_default_kwargs
    )



from .rovpp_v2_journal_lite_simple_as import ROVPPV2JournalLiteSimpleAS
from .rovpp_v2_journal_simple_as import ROVPPV2JournalSimpleAS

__all__ = ["ROVPPV2JournalLiteSimpleAS", "ROVPPV2JournalSimpleAS"]



from frozendict import frozendict

from bgpy.tests.engine_tests.graphs import graph_048
from bgpy.tests.engine_tests.utils import EngineTestConfig

from bgpy.simulation_engine import BGPSimpleAS
from bgpy.enums import ASNs
from bgpy.simulation_framework import ScenarioConfig, NonRoutedSuperprefixPrefixHijack

from rovpp import ROVPPAnn, ROVPPV2aSimpleAS

config_218 = EngineTestConfig(
    name="218",
    desc="Superprefix+Prefix Attack on NonRouted Prefix with v2a",
    scenario_config=ScenarioConfig(
        ScenarioCls=NonRoutedSuperprefixPrefixHijack,
        BaseASCls=BGPSimpleAS,
        AdoptASCls=ROVPPV2aSimpleAS,
        AnnCls=ROVPPAnn,
        override_attacker_asns=frozenset({ASNs.ATTACKER.value}),
        override_victim_asns=frozenset({ASNs.VICTIM.value}),
        override_non_default_asn_cls_dict=frozendict(
            {
                3: ROVPPV2aSimpleAS,
                4: ROVPPV2aSimpleAS,
                6: ROVPPV2aSimpleAS,
            }
        ),
    ),
    graph=graph_048,
    propagation_rounds=1,
)



from typing import Dict, Tuple

from frozendict import frozendict

from bgpy.simulation_engine.announcement import Announcement as Ann  # noqa
from bgpy import ROVSimpleAS
from bgpy import Relationships
from bgpy import Scenario
from bgpy import NonRoutedSuperprefixHijack

from ...rovpp_ann import ROVPPAnn


class ROVPPV1LiteSimpleAS(ROVSimpleAS):
    name = "ROV++V1 Lite Simple"

    def __init__(self, *args, **kwargs):
        super(ROVPPV1LiteSimpleAS, self).__init__(*args, **kwargs)
        self.temp_holes = frozendict()

    def _policy_propagate(self, _, ann, *args) -> bool:
        """Only propagate announcements that aren't blackholes"""

        # Policy handled this ann for propagation (and did nothing)
        return bool(ann.blackhole)

    def receive_ann(self, ann: Ann, *args, **kwargs):  # type: ignore
        """Ensures that announcments are ROV++ and valid"""

        if not isinstance(ann, ROVPPAnn):
            raise NotImplementedError("Not an ROV++ Announcement")
        return super(ROVPPV1LiteSimpleAS, self).receive_ann(ann, *args, **kwargs)

    # Mypy errors out when getting subclass of this
    def process_incoming_anns(
        self,  # type: ignore
        *,
        from_rel: Relationships,
        propagation_round: int,
        scenario: "Scenario",
        reset_q: bool = True
    ):
        """Processes all incoming announcements"""

        # Super janky mutability garbage. Should be fixed later.
        self.temp_holes: Dict[
            Ann, Tuple[Ann]  # type: ignore
        ] = self._get_ann_to_holes_dict(scenario)
        super(ROVPPV1LiteSimpleAS, self).process_incoming_anns(
            from_rel=from_rel,
            propagation_round=propagation_round,
            scenario=scenario,
            reset_q=False,
        )
        self._add_blackholes(self.temp_holes, from_rel, scenario)

        # It's possible that we had a previously valid prefix
        # Then later recieved a subprefix that was invalid
        # So we must recount the holes of each ann in local RIB
        self._recount_holes(propagation_round)

        self._reset_q(reset_q)

    def _recount_holes(self, propagation_round):
        # It's possible that we had a previously valid prefix
        # Then later recieved a subprefix that was invalid
        # Or there was previously an invalid subprefix
        # But later that invalid subprefix was removed
        # So we must recount the holes of each ann in local RIB
        assert propagation_round == 0, "Must recount holes if you plan on this"

    # mypy can't subclass properly
    def _reset_q(self, reset_q: bool):  # type: ignore
        if reset_q:
            self.temp_holes = dict()
        super(ROVPPV1LiteSimpleAS, self)._reset_q(reset_q)

    def _get_ann_to_holes_dict(self, scenario: "Scenario"):
        """Gets announcements to a typle of Ann holes

        Holes are subprefix hijacks
        """

        holes = dict()
        for _, ann_list in self._recv_q.prefix_anns():
            for ann in ann_list:
                ann_holes = []
                for subprefix in scenario.ordered_prefix_subprefix_dict[ann.prefix]:
                    for sub_ann in self._recv_q.get_ann_list(subprefix):
                        # Holes are only from same neighbor
                        if (
                            sub_ann.invalid_by_roa
                            and sub_ann.as_path[0] == ann.as_path[0]
                        ):
                            ann_holes.append(sub_ann)
                # check all non routed announcements of the same prefix
                for same_prefix_ann in self._recv_q.get_ann_list(ann.prefix):
                    if (
                        same_prefix_ann.invalid_by_roa
                        and not same_prefix_ann.roa_routed
                        and same_prefix_ann.as_path[0] == ann.as_path[0]
                    ):
                        ann_holes.append(same_prefix_ann)
                holes[ann] = tuple(ann_holes)
        return frozendict(holes)

    def _add_blackholes(self, holes, from_rel, scenario):
        """Manipulates local RIB by adding blackholes and dropping invalid"""

        blackholes_to_add = []
        local_rib_to_remove = list()
        # For each ann in local RIB:
        for _, ann in self._local_rib.prefix_anns():
            # For each hole in ann: (holes are invalid subprefixes)
            for unprocessed_hole_ann in ann.holes:
                # If there is not an existing valid ann for that hole subprefix
                existing_local_rib_subprefix_ann = self._local_rib.get_ann(
                    unprocessed_hole_ann.prefix
                )

                if existing_local_rib_subprefix_ann is None or (
                    existing_local_rib_subprefix_ann.invalid_by_roa
                    and not existing_local_rib_subprefix_ann.preventive
                    # Without this line
                    # The same local rib Ann will try to create another
                    # blackhole for each from_rel
                    # But we don't want it to recreate
                    # And for single round prop, a future valid ann won't
                    # override the current valid ann due to gao rexford
                    and not existing_local_rib_subprefix_ann.blackhole
                ):
                    # If another entry exists, remove it
                    if self._local_rib.get_ann(unprocessed_hole_ann.prefix):
                        # Remove current ann and replace with blackhole
                        # These lines will only get hit if a subclass accepts
                        # invalid announcements and considers them valid
                        # which doesn't make sense, so won't bother testing
                        local_rib_to_remove.append(unprocessed_hole_ann.prefix)
                    # Create the blackhole
                    blackhole = self._copy_and_process(
                        unprocessed_hole_ann,
                        # Hole anns aren't processed, whereas ann's are,
                        # so anns have the correct relationship
                        ann.recv_relationship,
                        overwrite_default_kwargs={
                            # Aug 24 2023 just changed this from holes
                            # (which is a dict of ann: holes)
                            # To only the holes that are relevant for the ann
                            # (which is holes[ann])
                            # If this causes an error, just set it to (),
                            # Since for this paper and these specific simulations,
                            # the holes in blackholes don't matter at all
                            "holes": holes[unprocessed_hole_ann],
                            "blackhole": True,
                            "traceback_end": True,
                        },
                    )

                    blackholes_to_add.append(blackhole)

        for prefix in local_rib_to_remove:
            self._local_rib.remove_ann(prefix)  # TODO TEST

        # Must also add announcements that never made it into the local rib
        # Because the professors want to blackhole announcements even if ROV
        # Would have otherwise dropped them
        # For each ann in local RIB:
        for prefix, ann_list in self._recv_q.prefix_anns():
            # Don't go over announcements you've already checked in local rib
            if self._local_rib.get_ann(prefix):
                continue
            for ann in ann_list:
                # Get the anns that aren't routed and add them
                if not ann.roa_routed and ann.invalid_by_roa:
                    # Create the blackhole
                    blackhole = self._copy_and_process(
                        ann,
                        from_rel,
                        overwrite_default_kwargs={
                            "holes": (),
                            "blackhole": True,
                            "traceback_end": True,
                        },
                    )

                    blackholes_to_add.append(blackhole)

        # Hardcoded to blackhole superprefixes that we know exist
        # Professors wanted this change after we finished,
        # not worth the time to add correctly and they okayed this
        if isinstance(scenario, NonRoutedSuperprefixHijack):
            ann = self._local_rib.get_ann("1.0.0.0/8")
            if ann:
                bhole = self._copy_and_process(
                    ann,
                    ann.recv_relationship,
                    overwrite_default_kwargs={
                        "prefix": "1.2.0.0/16",
                        "holes": (),
                        "blackhole": True,
                        "traceback_end": True,
                    },
                )
                blackholes_to_add.append(bhole)
            # Add this new prefix to the dict for traceback
            # Keys must also be in order omg
            scenario.ordered_prefix_subprefix_dict = {
                "1.2.0.0/16": [],
                "1.0.0.0/8": ["1.2.0.0/16"],
            }

        # Do this here to avoid changing dict size
        for blackhole in blackholes_to_add:
            # Add the blackhole
            self._local_rib.add_ann(blackhole)
            # Do nothing - ann should already be a blackhole
            assert (
                blackhole.blackhole and blackhole.invalid_by_roa
            ) or not blackhole.invalid_by_roa

    def _copy_and_process(self, ann, recv_relationship, overwrite_default_kwargs=None):
        """Deep copies ann and modifies attrs"""

        if overwrite_default_kwargs:  # noqa
            # This is a no op for us, but for future subclasses it might
            # encounter this line of code. Omitting coverage to save time
            if "holes" not in overwrite_default_kwargs:  # pragma: no cover
                overwrite_default_kwargs["holes"] = self.temp_holes[ann]
        else:
            overwrite_default_kwargs = {"holes": self.temp_holes[ann]}

        return super(ROVPPV1LiteSimpleAS, self)._copy_and_process(
            ann, recv_relationship, overwrite_default_kwargs=overwrite_default_kwargs
        )

    def _process_outgoing_ann(self, neighbor, ann, *args, **kwargs):
        no_holes_ann = ann.copy(overwrite_default_kwargs={"holes": ()})
        super(ROVPPV1LiteSimpleAS, self)._process_outgoing_ann(
            neighbor, no_holes_ann, *args, **kwargs
        )



from dataclasses import dataclass, asdict
from typing import Tuple

from frozendict import frozendict
from yamlable import yaml_info

from bgpy import Announcement


@yaml_info(yaml_tag="ROVPPAnn")
@dataclass(slots=True, frozen=True)
class ROVPPAnn(Announcement):
    holes: Tuple[str] = ()
    blackhole: bool = False
    preventive: bool = False
    attacker_on_route: bool = False

    def __to_yaml_dict__(self):
        """This optional method is called when you call yaml.dump()

        We need to overwrite this from the base class because the holes attr
        contains a dictionary so we need to do this
        """

        dct = asdict(self)
        return frozendict(dct)



from frozendict import frozendict

from bgpy.tests.engine_tests.graphs import graph_038
from bgpy.tests.engine_tests.utils import EngineTestConfig

from bgpy.simulation_engine import BGPSimpleAS
from bgpy.enums import ASNs
from bgpy.simulation_framework import ScenarioConfig, SubprefixHijack

from rovpp import ROVPPAnn, ROVPPV1LiteSimpleAS


config_069 = EngineTestConfig(
    name="069",
    desc=(
        "ROV++ v1 lite adopting\n"
        "Example where v1 leads a higher disconnection rate\nand "
        "lower successful connection rate\nthan v2 (for non-adopting "
        "ASes).\nAS 57 is the legit origin (of 1.2/16),\n666 "
        "does subprefix as usual.\nAS 33 and AS 1 are adopters\n"
        "(of ROV++ , v1 or v2 "
        "as desired).\nIf 33 and 1 adopt v1, then 33 blackholes 1.2.3/24 "
        "and announces 1.2/16 to 1, with AS path of 33-34-57.\nSo AS 1 "
        "prefers this (shorter) path, cf to the "
        "alternative: 11-12-135-57.\nThis causes disconnection by the "
        "blackhole that was set up at 33.\n"
        "If 33 and 1 adopt v2,\nthen AS 1 will receive the blackhole "
        "announcement from 33\nand hence prefer the hole-free route via 11"
        "\n(which only has announcement 1.2/16),\nand hence route "
        "the traffic correctly to the origin.\nIf AS 33 and AS 1 "
        "use v1 Lite,\nthen "
        "AS 1 will still use the path via AS 33,\nand hence will "
        "have the same results as those when the policy is v1."
    ),
    scenario_config=ScenarioConfig(
        ScenarioCls=SubprefixHijack,
        BaseASCls=BGPSimpleAS,
        AdoptASCls=ROVPPV1LiteSimpleAS,
        override_attacker_asns=frozenset({ASNs.ATTACKER.value}),
        override_victim_asns=frozenset({ASNs.VICTIM.value}),
        override_non_default_asn_cls_dict=frozendict(
            {
                1: ROVPPV1LiteSimpleAS,
                33: ROVPPV1LiteSimpleAS,
            }
        ),
        AnnCls=ROVPPAnn,
    ),
    graph=graph_038,
)



from frozendict import frozendict

from bgpy.tests.engine_tests.graphs import graph_036
from bgpy.tests.engine_tests.utils import EngineTestConfig

from bgpy.simulation_engine import BGPSimpleAS
from bgpy.enums import ASNs
from bgpy.simulation_framework import ScenarioConfig, SubprefixHijack

from rovpp import ROVPPAnn
from rovpp import ROVPPV2aSimpleAS

config_174 = EngineTestConfig(
    name="174",
    desc="Subprefix Hijack Attack with v2a",
    scenario_config=ScenarioConfig(
        ScenarioCls=SubprefixHijack,
        BaseASCls=BGPSimpleAS,
        AdoptASCls=ROVPPV2aSimpleAS,
        AnnCls=ROVPPAnn,
        override_attacker_asns=frozenset({ASNs.ATTACKER.value}),
        override_victim_asns=frozenset({ASNs.VICTIM.value}),
        override_non_default_asn_cls_dict=frozendict(
            {
                5: ROVPPV2aSimpleAS,
                8: ROVPPV2aSimpleAS,
                10: ROVPPV2aSimpleAS,
                15: ROVPPV2aSimpleAS,
                ASNs.VICTIM.value: ROVPPV2aSimpleAS,
            }
        ),
    ),
    graph=graph_036,
    propagation_rounds=1,
)



from pathlib import Path

import pytest

from bgpy import EngineTester
from bgpy import EngineTestConfig
from bgpy import engine_test_configs as bgpy_engine_test_configs

from .engine_test_configs import engine_test_configs


@pytest.mark.engine
class TestEngine:
    """Performs a system test on the engine

    See README for in depth details
    """

    @pytest.mark.parametrize("conf", bgpy_engine_test_configs + engine_test_configs)
    def test_engine(self, conf: EngineTestConfig, overwrite: bool):
        """Performs a system test on the engine

        See README for in depth details
        """

        EngineTester(
            base_dir=self.base_dir, conf=conf, overwrite=overwrite
        ).test_engine()

    @property
    def base_dir(self) -> Path:
        """Returns test output dir"""

        return Path(__file__).parent / "engine_test_outputs"



from frozendict import frozendict

from bgpy.tests.engine_tests.graphs import graph_004
from bgpy.tests.engine_tests.utils import EngineTestConfig
from bgpy.simulation_engine import BGPSimpleAS, ROVSimpleAS
from bgpy.enums import ASNs
from bgpy.simulation_framework import ScenarioConfig, SubprefixHijack

from rovpp import ROVPPV2aLiteSimpleAS, ROVPPAnn

config_042 = EngineTestConfig(
    name="042",
    desc=(
        "Subprefix Hijack from fig 3a in paper with ROV++ v2a lite "
        "adopting at AS 4 and 8, and ROV at 7. "
        "ASes 4, 8, and 7 should be disconnected"
    ),
    scenario_config=ScenarioConfig(
        ScenarioCls=SubprefixHijack,
        BaseASCls=BGPSimpleAS,
        AdoptASCls=ROVPPV2aLiteSimpleAS,
        AnnCls=ROVPPAnn,
        override_attacker_asns=frozenset({ASNs.ATTACKER.value}),
        override_victim_asns=frozenset({ASNs.VICTIM.value}),
        override_non_default_asn_cls_dict=frozendict(
            {4: ROVPPV2aLiteSimpleAS, 7: ROVSimpleAS, 8: ROVPPV2aLiteSimpleAS}
        ),
    ),
    graph=graph_004,
)



from bgpy import Announcement
from bgpy import BGPAS, Prefixes, Relationships, ROVAS
from bgpy import Scenario, SubprefixHijack

from .non_lite import NonLite
from .v2 import ROVPPV2LiteSimpleAS
from .v2 import ROVPPV2SimpleAS
from .v1 import ROVPPV1LiteSimpleAS


class ROVPPV3AS(NonLite, ROVAS, ROVPPV2LiteSimpleAS):
    """If a customer announces a valid route, don't send preventive

    If you send a preventive,
            must add attacker on route flag to the valid route
        If later the attack is withdrawn, must withdraw this preventive
    Only send preventives to customers
    Anns with attacker on route flag are
        not allowed to be chosen as safe routes
    """

    name = "ROV++V3"

    # mypy doesn't understand parent/subclass here
    def _policy_propagate(
        self, neighbor, ann: Announcement, propagate_to, send_rels  # type: ignore
    ):
        """Special cases for propagation handled by the V3 policy

        Blackholes propagate normally
        preventives must:
            Only be sent to customers
            Don't send if there was a valid route from that neighbor
        """

        if ann.blackhole:
            # For now do V1
            # return True
            # Is this calling the proper func??
            return ROVPPV2SimpleAS._policy_propagate(
                self, neighbor, ann, propagate_to, send_rels
            )
        # NOTE that preventive is the subprefix.
        #    The valid route isn't considered preventive
        elif ann.preventive:
            # Same logic as V2. Don't override because V2 uses this lol
            if self._send_competing_hijack_allowed(ann, propagate_to):
                # input("\n" * 3 + "allowed to be sent" + "\n" * 3)
                # Make sure you don't send to a neighbor
                # that sent a valid route
                # NOTE: that includes neighbors
                # that sent valid route and subprefix
                # imo these two if statements
                # make this policy almost never occur
                safe_route_prefix = Prefixes.PREFIX.value
                if not self._neighbor_sent_valid_ann(neighbor, safe_route_prefix):
                    return False
                    # self._process_outgoing_ann(neighbor,
                    #                           ann,
                    #                            propagate_to,
                    #                           send_rels)
                # else:
                #    # Send that neighbor a blackhole??????????
                #    # On second thought nm, because your sending to customers
                #    # So if customer sent subprefix,
                #    # you will not replace it (Gao rexford)
            # You should always be able to send competing hijack for
            # for preventive, since they should only send to peer/provider
            # Because of this, this line should never be reached
            # unless there is a V3 attack, however, since no one should
            # ever ever deploy v3, we don't need to test for this
            return True  # TODO TEST
        else:
            return False

    # Ann here is the subprefix since it is a preventive
    def _neighbor_sent_valid_ann(self, neighbor, safe_route_prefix):
        neighbor_info = self._ribs_in.get_unprocessed_ann_recv_rel(
            neighbor.asn, safe_route_prefix
        )

        return neighbor_info is not None

    # Mypy doesn't understand the subclassing
    def process_incoming_anns(
        self,  # type: ignore
        *,
        from_rel: Relationships,
        propagation_round: int,
        scenario: "Scenario",
        reset_q: bool = True
    ):
        """Process all announcements that were incoming from a specific rel"""

        err = (
            "This entire policy hardcoded for a victim prefix "
            "and attacker subprefix"
            " Not going to make this dynamic "
            "because this is a useless policy"
        )
        assert isinstance(scenario, SubprefixHijack), err

        # Holes are invalid subprefixes from the same neighbor
        # We do this here so that we can optimize it
        # Or else it is much too slow to have a generalized version
        # See ROVPP_v1_Lite_slow, it has like 5+ nested for loops

        self.temp_holes = self._get_ann_to_holes_dict(scenario)

        super().process_incoming_anns(
            from_rel=from_rel,
            propagation_round=propagation_round,
            scenario=scenario,
            reset_q=False,
        )
        self._get_and_assign_preventives(self.temp_holes, from_rel)

        self._add_blackholes(self.temp_holes, from_rel, scenario)

        # Move holes from temp_holes and resets q
        self._reset_q(reset_q)

    def _reset_q(self, reset_q: bool):  # type: ignore
        if reset_q:
            self.temp_holes = dict()
        super(ROVPPV1LiteSimpleAS, self)._reset_q(reset_q)

    def _get_and_assign_preventives(self, holes, recv_relationship):
        """
        Conditions to create a preventive:
            Check the prefix in the local rib.
                If it has no holes, and no attacker on route flag,
                    this is a safe route

        When creating preventives:
            Modify valid route in your local rib to have attacker on route flag
            Create a subprefix that is a copy of the valid ann
                but with subprefix
                NOTE: also must make roa validity invalid!!
        """

        # Check if we have a safe alternate (no holes, not attacker on route)
        victim_ann = self._local_rib.get_ann(Prefixes.PREFIX.value)

        # Attacker is on the route
        if victim_ann is not None and len(victim_ann.holes) > 0:
            self._local_rib.add_ann(
                victim_ann.copy(overwrite_default_kwargs={"attacker_on_route": True})
            )
        # Safe route - ths condition plus not the two conditions above
        elif victim_ann is not None and victim_ann.attacker_on_route is False:
            # Must do this here, since we don't want to create
            # preventives we won't send
            # Because we should instead create blackholes for that case
            if self._recv_hijack_from_peer_provider():
                # TODO Create the preventive ann and store that in local rib
                preventive_ann = victim_ann.copy(
                    overwrite_default_kwargs={
                        "prefix": Prefixes.SUBPREFIX.value,
                        "roa_valid_length": False,
                        "preventive": True,
                        # This covers the case when victim_ann has no holes
                        # But we are sending a preventivei
                        # ?????????????????????????????????????????????????
                        # Do we need this line?????????????????????????????
                        "attacker_on_route": True,
                        "holes": tuple(),
                    }
                )

                self._local_rib.add_ann(preventive_ann)

    def _recv_hijack_from_peer_provider(self):
        """Did we recieve a hijack at any point from a peer or provider??"""

        for ann_info in self._ribs_in.get_ann_infos(Prefixes.SUBPREFIX.value):
            # assert ann_info.unprocessed_ann is not None, "for mypy"
            if (
                ann_info.recv_relationship
                in [Relationships.PEERS, Relationships.PROVIDERS]
                and ann_info.unprocessed_ann.invalid_by_roa  # type: ignore
                and not ann_info.unprocessed_ann.preventive  # type: ignore
            ):  # noqa
                return True

        return False

    def _valid_ann(self, ann: Announcement, *args, **kwargs) -> bool:  # type: ignore
        if ann.invalid_by_roa and not ann.preventive:
            return False
        else:
            return bool(BGPAS._valid_ann(self, ann, *args, **kwargs))

    def _copy_and_process(self, ann, recv_relationship, overwrite_default_kwargs=None):
        """Deep copies ann and modifies attrs"""

        if overwrite_default_kwargs:
            overwrite_default_kwargs["holes"] = self.temp_holes[ann]
        else:
            overwrite_default_kwargs = {"holes": self.temp_holes[ann]}

        return super(ROVPPV1LiteSimpleAS, self)._copy_and_process(
            ann, recv_relationship, overwrite_default_kwargs=overwrite_default_kwargs
        )



from frozendict import frozendict

from bgpy.tests.engine_tests.graphs import graph_005
from bgpy.tests.engine_tests.utils import EngineTestConfig
from bgpy.simulation_engine import BGPSimpleAS
from bgpy.enums import ASNs
from bgpy.simulation_framework import ScenarioConfig, SubprefixHijack

from rovpp import ROVPPV2SimpleAS, ROVPPAnn

config_044 = EngineTestConfig(
    name="044",
    desc="Subprefix Hijack from fig 4 in paper with ROV++ v2",
    scenario_config=ScenarioConfig(
        ScenarioCls=SubprefixHijack,
        BaseASCls=BGPSimpleAS,
        AdoptASCls=ROVPPV2SimpleAS,
        AnnCls=ROVPPAnn,
        override_attacker_asns=frozenset({ASNs.ATTACKER.value}),
        override_victim_asns=frozenset({ASNs.VICTIM.value}),
        override_non_default_asn_cls_dict=frozendict({5: ROVPPV2SimpleAS}),
    ),
    graph=graph_005,
)



from typing import Optional

from bgpy import Announcement as Ann
from bgpy import BGPSimpleAS
from bgpy import Relationships


class NonLite:
    """Do nothing for new holes better funcs (Lite)"""

    def _new_ann_better(
        self,  # type: ignore
        current_ann: Optional[Ann],
        current_processed: bool,
        default_current_recv_rel: Relationships,
        new_ann: Ann,
        new_processed: Relationships,
        default_new_recv_rel: Relationships,
    ) -> bool:
        """Must include this here since we blackhole prefixes now

        This does the same thing as the original func
        except it deprefers blackholes and invalid anns
        """

        if current_ann is None:
            assert new_ann is not None
            return True
        # Valid > invalid
        new_validity_better = self._new_validity_better(current_ann, new_ann)
        # This is a no op without specific subclasses, we can ignore
        # for the purposes of our paper
        if new_validity_better is not None:
            return new_validity_better  # pragma: no cover
        else:
            # Not blackhole > blackhole
            new_blackhole_state_better = self._new_blackhole_state_better(
                current_ann, new_ann
            )
            # This is a no unless you create a class that considers
            # both blackholes and preventives valid, which we do not
            # So we can ignore for this paper
            if new_blackhole_state_better is not None:
                return new_blackhole_state_better  # pragma: no cover
            else:
                # First check if new relationship is better
                new_rel_better: Optional[bool] = BGPSimpleAS._new_rel_better(
                    self,
                    current_ann,
                    current_processed,
                    default_current_recv_rel,
                    new_ann,
                    new_processed,
                    default_new_recv_rel,
                )
                # If new rel better is True or False, return it
                if new_rel_better is not None:
                    return new_rel_better
                else:
                    new_holes_better = self._new_holes_better(
                        current_ann, current_processed, new_ann, new_processed
                    )
                    if new_holes_better is not None:
                        return new_holes_better
                    else:
                        # Return the outcome of as path and tiebreaks
                        # mypy doesn't recognize that this is always a bool
                        return BGPSimpleAS._new_as_path_ties_better(
                            self,
                            current_ann,  # type: ignore
                            current_processed,
                            new_ann,
                            new_processed,
                        )

    def _new_validity_better(self, current_ann, new_ann) -> Optional[bool]:
        """Returns True if new better, False if old better, None if eq"""

        if not new_ann.invalid_by_roa and current_ann.invalid_by_roa:
            # This only occurs in subclasses we do not implement
            # so we will ignore it to save time
            return True  # pragma: no cover
        elif new_ann.invalid_by_roa and not current_ann.invalid_by_roa:
            # This only occurs in subclasses we do not implement
            # so we will ignore it to save time
            return False  # pragma: no cover
        else:
            return None

    def _new_blackhole_state_better(self, current_ann, new_ann) -> Optional[bool]:
        """Returns True if new better, False if old better, None if eq

        This function is basically a no op unless you create a subclass
        that doesn't drop invalid announcements for some reason and also
        keeps preventives and blackholes. To save time for this research
        paper, we omit the coverage
        """

        if not new_ann.blackhole and current_ann.blackhole:
            return True  # pragma: no cover
        elif new_ann.blackhole and not current_ann.blackhole:
            return False  # pragma: no cover
        # Preventives > blackholes
        elif new_ann.preventive and current_ann.blackhole:
            return True  # pragma: no cover
        else:
            return None

    def _new_holes_better(
        self, current_ann, current_ann_processed, new_ann, new_ann_processed
    ) -> Optional[bool]:
        """Returns new ann has less holes, or None if =="""

        # Could do this using int(processed) but so unreadable

        # Holes for new announcement
        if new_ann_processed:
            # This only occurs in subclasses we do not implement
            # so we will ignore it to save time
            new_holes = len(new_ann.holes)  # pragma: no cover
        else:
            new_holes = len(self.temp_holes[new_ann])  # type: ignore

        # Holes for current announcement
        if current_ann_processed:
            # This only occurs in subclasses we do not implement
            # so we ill ignore it to save time
            current_holes = len(current_ann.holes)  # pragma: no cover
        else:
            current_holes = len(self.temp_holes[current_ann])  # type: ignore

        if new_holes < current_holes:
            return True
        elif new_holes > current_holes:
            return False
        # B explicit for future devs
        # One is not better than the other, return None
        else:
            return None



from .rovpp_v2_shorten_lite_simple_as import ROVPPV2ShortenLiteSimpleAS
from .rovpp_v2_shorten_simple_as import ROVPPV2ShortenSimpleAS

__all__ = ["ROVPPV2ShortenLiteSimpleAS", "ROVPPV2ShortenSimpleAS"]



from frozendict import frozendict

from bgpy.tests.engine_tests.graphs import graph_003
from bgpy.tests.engine_tests.utils import EngineTestConfig
from bgpy.simulation_engine import BGPSimpleAS, ROVSimpleAS
from bgpy.enums import ASNs
from bgpy.simulation_framework import ScenarioConfig, SubprefixHijack

config_035 = EngineTestConfig(
    name="rovpp_035",
    desc="Subprefix Hijack from fig 2 in paper with ROV adopting.",
    scenario_config=ScenarioConfig(
        ScenarioCls=SubprefixHijack,
        BaseASCls=BGPSimpleAS,
        AdoptASCls=ROVSimpleAS,
        override_attacker_asns=frozenset({ASNs.ATTACKER.value}),
        override_victim_asns=frozenset({ASNs.VICTIM.value}),
        override_non_default_asn_cls_dict=frozendict({3: ROVSimpleAS, 4: ROVSimpleAS}),
    ),
    graph=graph_003,
)



from argparse import ArgumentParser
from datetime import datetime
from multiprocessing import Process, cpu_count
from pathlib import Path
import time

from bgpy import BGPSimpleAS
from bgpy import ROVSimpleAS
from bgpy import Simulation
from bgpy import SpecialPercentAdoptions  # noqa
from bgpy import ScenarioConfig

# Hijacks
from bgpy import SubprefixHijack
from bgpy import NonRoutedSuperprefixHijack
from bgpy import NonRoutedSuperprefixPrefixHijack
from bgpy import SuperprefixPrefixHijack
from bgpy import PrefixHijack
from bgpy import NonRoutedPrefixHijack

# Mixed deployment
from bgpy import get_real_world_rov_asn_cls_dict

# LITE
from .as_classes import ROVPPV1LiteSimpleAS

# NON LITE
from .as_classes import ROVPPV1SimpleAS
from .as_classes import ROVPPV2SimpleAS
from .as_classes import ROVPPV2aSimpleAS
from .as_classes import ROVPPV2ShortenSimpleAS
from .as_classes import ROVPPV2JournalSimpleAS
from .as_classes import ROVPPV3AS

from .rovpp_ann import ROVPPAnn


BASE_PATH = Path("~/Desktop/graphs/").expanduser()


def get_default_kwargs(quick, trials=None):  # pragma: no cover
    if not trials:
        trials = 1 if quick else 1000
    if quick:
        return {
            "percent_adoptions": (0.5,),
            "num_trials": trials,
            "parse_cpus": 1,
        }
    else:  # pragma: no cover
        return {
            "percent_adoptions": (
                0.001,  # SpecialPercentAdoptions.ONLY_ONE,  # .01,
                # 0.5,
                0.1,
                0.2,
                0.3,
                0.4,
                0.6,
                0.8,
                # .99],
                0.998,  # SpecialPercentAdoptions.ALL_BUT_ONE,
            ),
            "num_trials": trials,
            "parse_cpus": cpu_count(),
            "caida_run_kwargs": {"dl_time": datetime(2023, 10, 5)},
            "python_hash_seed": 0,
        }


ROV_NON_LITE_ROVPP = (
    ROVSimpleAS,
    ROVPPV1SimpleAS,
    ROVPPV2JournalSimpleAS,
    # ROVPPV2aSimpleAS,
)


class V1Multi1(ROVPPV1SimpleAS):
    name = "ROV++V1 1 Attacker"


class V1Multi10(ROVPPV1SimpleAS):
    name = "ROV++V1 10 Attackers"


class V1Multi100(ROVPPV1SimpleAS):
    name = "ROV++V1 100 Attackers"


MULTI_ATK_AS_CLASSES = (V1Multi1, V1Multi10, V1Multi100)


def run_simulation(sim,):
    """See BGPy FAQ

    Long story short Each sim leaks memory due to some weird PyPy garbage collection
    that doesn't occur in Python. Running sims one after the other (for now)
    requires this. In the future we'll move to Rust, which has proper memory
    management.
    """

    print("Starting simulation in a new process")
    start = time.perf_counter()
    sim.run(
        graph_factory_kwargs={
            "label_replacement_dict": {
                # BGPy
                BGPSimpleAS.name: "BGP",
                ROVSimpleAS.name: "ROV",
                # ROV++
                ROVPPV1LiteSimpleAS.name: "ROV++V1 Lite",
                # NON LITE.name: "",
                ROVPPV1SimpleAS.name: "ROV++V1",
                ROVPPV2SimpleAS.name: "ROV++V2 No Customers",
                ROVPPV2aSimpleAS.name: "ROV++V2 Aggressive",
                ROVPPV2ShortenSimpleAS.name: "ROV++V2 Shorten",
                ROVPPV2JournalSimpleAS.name: "ROV++V2",
                ROVPPV3AS.name: "ROV++V3",
            },
            "y_axis_label_replacement_dict": {
                "PERCENT ATTACKER SUCCESS": "Data Plane % Hijacked",
                "PERCENT VICTIM SUCCESS": "Data Plane % Successfully Connected",
                "PERCENT DISCONNECTED": "Data Plane % Disconnected",
            },
            "x_axis_label_replacement_dict": {},
        }
    )
    print(f"{sim.output_dir} {(time.perf_counter() - start)}")


# Ignoring coverage on this func because it would cause every line
# to be covered, and there is a bare bones system test that just runs
# through these
def main(quick=True, trials=1, graph_index=None):  # pragma: no cover
    # assert isinstance(input("Turn asserts off for speed?"), str)
    sims = [
        Simulation(
            scenario_configs=tuple(
                [
                    ScenarioConfig(
                        ScenarioCls=SubprefixHijack, AdoptASCls=Cls, AnnCls=ROVPPAnn
                    )
                    for Cls in ROV_NON_LITE_ROVPP + (ROVPPV3AS,)
                ]
            ),
            output_dir=BASE_PATH / "subprefix",
            **get_default_kwargs(quick=quick, trials=trials),
        ),
        Simulation(
            scenario_configs=tuple(
                [
                    ScenarioConfig(
                        ScenarioCls=SubprefixHijack,
                        AdoptASCls=ASCls,
                        AnnCls=ROVPPAnn,
                        num_attackers=num_attackers,
                    )
                    for (ASCls, num_attackers) in zip(
                        MULTI_ATK_AS_CLASSES, (1, 10, 100)
                    )
                ]
            ),
            output_dir=BASE_PATH / "subprefix_multi_atk",
            **get_default_kwargs(quick=quick, trials=trials),
        ),
        Simulation(
            scenario_configs=tuple(
                [
                    ScenarioConfig(
                        ScenarioCls=NonRoutedPrefixHijack,
                        AdoptASCls=Cls,
                        AnnCls=ROVPPAnn,
                    )
                    for Cls in (
                        ROVPPV2SimpleAS,
                        ROVPPV2aSimpleAS,
                        ROVPPV2ShortenSimpleAS,
                        ROVPPV2JournalSimpleAS,
                    )
                ]
            ),
            output_dir=BASE_PATH / "v2_variants",
            **get_default_kwargs(quick=quick, trials=trials),
        ),
        Simulation(
            scenario_configs=tuple(
                [
                    ScenarioConfig(
                        ScenarioCls=SubprefixHijack,
                        AdoptASCls=Cls,
                        AnnCls=ROVPPAnn,
                        hardcoded_asn_cls_dict=get_real_world_rov_asn_cls_dict(
                            min_rov_confidence=0
                        ),
                    )
                    for Cls in ROV_NON_LITE_ROVPP + (ROVPPV3AS,)
                ]
            ),
            output_dir=BASE_PATH / "mixed_deployment",
            **get_default_kwargs(quick=quick, trials=trials),
        ),
        Simulation(
            scenario_configs=tuple(
                [
                    ScenarioConfig(
                        ScenarioCls=NonRoutedSuperprefixHijack,
                        AdoptASCls=Cls,
                        AnnCls=ROVPPAnn,
                    )
                    for Cls in ROV_NON_LITE_ROVPP
                ]
            ),
            output_dir=BASE_PATH / "non_routed_superprefix",
            **get_default_kwargs(quick=quick, trials=trials),
        ),
        Simulation(
            scenario_configs=tuple(
                [
                    ScenarioConfig(
                        ScenarioCls=SuperprefixPrefixHijack,
                        AdoptASCls=Cls,
                        AnnCls=ROVPPAnn,
                    )
                    for Cls in ROV_NON_LITE_ROVPP
                ]
            ),
            output_dir=BASE_PATH / "superprefix_prefix",
            **get_default_kwargs(quick=quick, trials=trials),
        ),
        Simulation(
            scenario_configs=tuple(
                [
                    ScenarioConfig(
                        ScenarioCls=PrefixHijack, AdoptASCls=Cls, AnnCls=ROVPPAnn
                    )
                    for Cls in ROV_NON_LITE_ROVPP
                ]
            ),
            output_dir=BASE_PATH / "prefix",
            **get_default_kwargs(quick=quick, trials=trials),
        ),
        Simulation(
            scenario_configs=tuple(
                [
                    ScenarioConfig(
                        ScenarioCls=NonRoutedPrefixHijack,
                        AdoptASCls=Cls,
                        AnnCls=ROVPPAnn,
                    )
                    for Cls in ROV_NON_LITE_ROVPP
                ]
            ),
            output_dir=BASE_PATH / "non_routed_prefix",
            **get_default_kwargs(quick=quick, trials=trials),
        ),
        Simulation(
            scenario_configs=tuple(
                [
                    ScenarioConfig(
                        ScenarioCls=NonRoutedSuperprefixPrefixHijack,
                        AdoptASCls=Cls,
                        AnnCls=ROVPPAnn,
                    )
                    for Cls in ROV_NON_LITE_ROVPP
                ]
            ),
            output_dir=BASE_PATH / "non_routed_superprefix_prefix",
            **get_default_kwargs(quick=quick, trials=trials),
        ),
        Simulation(
            scenario_configs=tuple(
                [
                    ScenarioConfig(
                        ScenarioCls=SubprefixHijack,
                        AdoptASCls=Cls,
                        AnnCls=ROVPPAnn,
                    )
                    for Cls in (ROVPPV1SimpleAS, ROVPPV1LiteSimpleAS)
                ]
            ),
            output_dir=BASE_PATH / "lite_vs_non_lite",
            **get_default_kwargs(quick=quick, trials=trials),
        ),
        Simulation(
            scenario_configs=tuple(
                [
                    ScenarioConfig(
                        ScenarioCls=SubprefixHijack,
                        AdoptASCls=Cls,
                        AnnCls=ROVPPAnn,
                    )
                    for Cls in (ROVSimpleAS,)
                ]
            ),
            output_dir=BASE_PATH / "ctrl_vs_data_plane",
            **get_default_kwargs(quick=quick, trials=trials),
        ),
    ]

    if graph_index is not None:
        sims = [sims[graph_index]]

    for sim in sims:
        if isinstance(sim, str):
            continue
        else:
            print("Spawning a sim process")
            # Create a Process for each simulation (see run_simulation for details)
            p = Process(target=run_simulation, args=(sim,))
            p.start()
            # Run one at a time due to resource constraints
            p.join()


if __name__ == "__main__":
    parser = ArgumentParser(description="Runs a simulation")
    parser.add_argument("--quick", dest="quick", default=False, action="store_true")
    parser.add_argument("--trials", type=int, dest="trials", default=1000)
    idx_default = -100
    parser.add_argument(
        "--graph_index", type=int, dest="graph_index", default=idx_default
    )
    args = parser.parse_args()
    graph_index = None if args.graph_index == idx_default else args.graph_index
    start = time.perf_counter()
    main(quick=args.quick, trials=args.trials, graph_index=graph_index)
    print((time.perf_counter() - start))



from .as_classes import ROVPPV1LiteSimpleAS
from .as_classes import ROVPPV2LiteSimpleAS
from .as_classes import ROVPPV2aLiteSimpleAS
from .as_classes import ROVPPV2ShortenLiteSimpleAS
from .as_classes import ROVPPV2JournalLiteSimpleAS
from .as_classes import ROVPPV1SimpleAS
from .as_classes import ROVPPV2SimpleAS
from .as_classes import ROVPPV2aSimpleAS
from .as_classes import ROVPPV2ShortenSimpleAS
from .as_classes import ROVPPV2JournalSimpleAS
from .as_classes import ROVPPV3AS

from .rovpp_ann import ROVPPAnn

__all__ = [
    "ROVPPV1LiteSimpleAS",
    "ROVPPV2LiteSimpleAS",
    "ROVPPV2aLiteSimpleAS",
    "ROVPPV2ShortenLiteSimpleAS",
    "ROVPPV2JournalLiteSimpleAS",
    "ROVPPV1SimpleAS",
    "ROVPPV2SimpleAS",
    "ROVPPV2aSimpleAS",
    "ROVPPV2ShortenSimpleAS",
    "ROVPPV2JournalSimpleAS",
    "ROVPPV3AS",
    "ROVPPAnn",
]



from bgpy import Relationships

from ..v2_base import ROVPPV2SimpleAS


class ROVPPV2JournalSimpleAS(ROVPPV2SimpleAS):
    name = "ROV++V2 Journal Simple"

    def _send_competing_hijack_allowed(self, ann, propagate_to):
        return (
            # Commenting this out, since Journal version also can get from customer
            # ann.recv_relationship
            # in [Relationships.PEERS, Relationships.PROVIDERS, Relationships.ORIGIN]
            # and propagate_to == Relationships.CUSTOMERS
            propagate_to == Relationships.CUSTOMERS
            and (not ann.roa_valid_length or not ann.roa_routed)
        )



from frozendict import frozendict

from bgpy.tests.engine_tests.graphs import graph_022
from bgpy.tests.engine_tests.utils import EngineTestConfig

from bgpy.simulation_engine import BGPSimpleAS
from bgpy.enums import ASNs
from bgpy.simulation_framework import ScenarioConfig, SubprefixHijack

from rovpp import ROVPPAnn, ROVPPV2aSimpleAS


config_059 = EngineTestConfig(
    name="059",
    desc=(
        "This is a v2a test.\nA suggested test for v2 versus v2a.\n"
        "v2a leads to additional "
        "benefits over v2.\nWhen AS 4 uses v2, AS 7 and its customer cone "
        "will be hijacked.\nWhen AS 4 uses v2a, the blackhole announcement"
        " from AS 4 will compete with the hijack announcement.\nIn this "
        "case, AS 7 will choose the blackhole announcement,\nand hence "
        "AS 7 and its customer cone\nwill be will be disconnected "
        "instead of hijacked."
    ),
    scenario_config=ScenarioConfig(
        ScenarioCls=SubprefixHijack,
        BaseASCls=BGPSimpleAS,
        AdoptASCls=ROVPPV2aSimpleAS,
        override_attacker_asns=frozenset({ASNs.ATTACKER.value}),
        override_victim_asns=frozenset({ASNs.VICTIM.value}),
        override_non_default_asn_cls_dict=frozendict({4: ROVPPV2aSimpleAS}),
        AnnCls=ROVPPAnn,
    ),
    graph=graph_022,
)



from bgpy import Relationships

from ...v1 import ROVPPV1LiteSimpleAS


class ROVPPV2LiteSimpleAS(ROVPPV1LiteSimpleAS):
    name = "ROV++V2_conference Lite Simple"

    def _policy_propagate(self, neighbor, ann, propagate_to, *args):
        """Deals with blackhole propagation

        If ann is a blackhole, it must be recv from peers/providers and must
        be sent only to customers
        """

        # If blackhole and subprefix or non-routed
        if ann.blackhole:
            if self._send_competing_hijack_allowed(ann, propagate_to):
                self._process_outgoing_ann(neighbor, ann, propagate_to, *args)
            return True
        else:
            return False

    def _send_competing_hijack_allowed(self, ann, propagate_to):
        return (
            ann.recv_relationship
            in [Relationships.PEERS, Relationships.PROVIDERS, Relationships.ORIGIN]
            and propagate_to == Relationships.CUSTOMERS
            and (not ann.roa_valid_length or not ann.roa_routed)
        )



from frozendict import frozendict

from bgpy.tests.engine_tests.graphs import graph_039
from bgpy.tests.engine_tests.utils import EngineTestConfig

from bgpy.simulation_engine import BGPSimpleAS
from bgpy.enums import ASNs
from bgpy.simulation_framework import ScenarioConfig, SubprefixHijack

from rovpp import ROVPPAnn, ROVPPV1LiteSimpleAS


config_072 = EngineTestConfig(
    name="072",
    desc=(
        "ROV++ v1 lite adopting\n"
        "Example where v2 leads to a higher disconnection "
        "rate\nand lower successful connection rate\n"
        "(for nonadopting ASes)."
        "When AS 33 adopts v2,\nit sends a blackhole announcement to "
        "AS 3,\nwhich will be further forwarded to AS 3’s customer"
        "cone.\nWhen AS 33 adopts v1,\nit only sets up a local "
        "blackhole.\nAS 3 only receives one announcement (1.2/16) "
        "from AS 1\nand"
        "no other announcement.\nIt will hence route traffic destining "
        "to 1.2.3/24 via AS 1 to the legit origin, AS 5.\n"
        "Similarly, AS 3’s "
        "customers’ traffic to 1.2.3/24 will be routed successfully.\n"
        "In the figure,\nwhether AS 5 adopts ROV++ or not doesn't matter\n"
        "since the hijack announcement will not be sent out by "
        "AS 5 to AS 1 anyway\n(due to valley-free routing)."
    ),
    scenario_config=ScenarioConfig(
        ScenarioCls=SubprefixHijack,
        BaseASCls=BGPSimpleAS,
        AdoptASCls=ROVPPV1LiteSimpleAS,
        override_attacker_asns=frozenset({ASNs.ATTACKER.value}),
        override_victim_asns=frozenset({ASNs.VICTIM.value}),
        override_non_default_asn_cls_dict=frozendict({33: ROVPPV1LiteSimpleAS}),
        AnnCls=ROVPPAnn,
    ),
    graph=graph_039,
)



from frozendict import frozendict

from bgpy.tests.engine_tests.graphs import graph_018
from bgpy.tests.engine_tests.utils import EngineTestConfig

from bgpy.simulation_engine import BGPSimpleAS
from bgpy.enums import ASNs
from bgpy.simulation_framework import ScenarioConfig, NonRoutedPrefixHijack

from rovpp import ROVPPAnn, ROVPPV2JournalSimpleAS

config_219 = EngineTestConfig(
    name="219",
    desc="Non routed prefix attack with v2_journal",
    scenario_config=ScenarioConfig(
        ScenarioCls=NonRoutedPrefixHijack,
        BaseASCls=BGPSimpleAS,
        AdoptASCls=ROVPPV2JournalSimpleAS,
        AnnCls=ROVPPAnn,
        override_attacker_asns=frozenset({3}),
        override_victim_asns=frozenset({ASNs.VICTIM.value}),
        override_non_default_asn_cls_dict=frozendict(
            {
                1: ROVPPV2JournalSimpleAS,
            }
        ),
    ),
    graph=graph_018,
    propagation_rounds=1,
)



from frozendict import frozendict

from bgpy.tests.engine_tests.graphs import graph_006
from bgpy.tests.engine_tests.utils import EngineTestConfig

from bgpy.simulation_engine import BGPSimpleAS
from bgpy.enums import ASNs
from bgpy.simulation_framework import ScenarioConfig, NonRoutedPrefixHijack

from rovpp import ROVPPV2SimpleAS, ROVPPAnn


config_052 = EngineTestConfig(
    name="052",
    desc=(
        "Prefix hijack on a non-routed prefix. Here adopting AS 2 is "
        "adopting ROV++ v2, and should detect the attack and create "
        "a blackhole announcement and send it to its customer."
    ),
    scenario_config=ScenarioConfig(
        ScenarioCls=NonRoutedPrefixHijack,
        BaseASCls=BGPSimpleAS,
        AdoptASCls=ROVPPV2SimpleAS,
        override_attacker_asns=frozenset({ASNs.ATTACKER.value}),
        override_victim_asns=frozenset({ASNs.VICTIM.value}),
        override_non_default_asn_cls_dict=frozendict({2: ROVPPV2SimpleAS}),
        AnnCls=ROVPPAnn,
    ),
    graph=graph_006,
)



from frozendict import frozendict

from bgpy.tests.engine_tests.graphs import graph_008
from bgpy.tests.engine_tests.utils import EngineTestConfig

from bgpy.simulation_engine import BGPSimpleAS
from bgpy.enums import ASNs
from bgpy.simulation_framework import ScenarioConfig, SubprefixHijack

from rovpp import ROVPPV2aLiteSimpleAS, ROVPPAnn


config_048 = EngineTestConfig(
    name="048",
    desc=(
        "Subprefix Hijack that tests blackhole announcements should "
        "be sent, even if hijack comes from a customer with ROV++ v2a "
        "lite. This comes with the added check to see if it gets sent to peers."
    ),
    scenario_config=ScenarioConfig(
        ScenarioCls=SubprefixHijack,
        BaseASCls=BGPSimpleAS,
        AdoptASCls=ROVPPV2aLiteSimpleAS,
        override_attacker_asns=frozenset({ASNs.ATTACKER.value}),
        override_victim_asns=frozenset({ASNs.VICTIM.value}),
        override_non_default_asn_cls_dict=frozendict({1: ROVPPV2aLiteSimpleAS}),
        AnnCls=ROVPPAnn,
    ),
    graph=graph_008,
)



from .config_035 import config_035
from .config_036 import config_036
from .config_037 import config_037
from .config_038 import config_038
from .config_039 import config_039
from .config_040 import config_040
from .config_041 import config_041
from .config_042 import config_042
from .config_043 import config_043
from .config_044 import config_044
from .config_045 import config_045
from .config_046 import config_046
from .config_047 import config_047
from .config_048 import config_048
from .config_049 import config_049
from .config_050 import config_050
from .config_051 import config_051
from .config_052 import config_052
from .config_053 import config_053
from .config_054 import config_054
from .config_055 import config_055
from .config_056 import config_056
from .config_057 import config_057
from .config_058 import config_058
from .config_059 import config_059
from .config_060 import config_060
from .config_061 import config_061
from .config_062 import config_062
from .config_063 import config_063
from .config_064 import config_064
from .config_065 import config_065
from .config_066 import config_066
from .config_067 import config_067
from .config_068 import config_068
from .config_069 import config_069
from .config_070 import config_070
from .config_071 import config_071
from .config_072 import config_072
from .config_073 import config_073
from .config_074 import config_074
from .config_075 import config_075
from .config_076 import config_076
from .config_077 import config_077
from .config_078 import config_078
from .config_079 import config_079
from .config_080 import config_080
from .config_081 import config_081
from .config_082 import config_082
from .config_083 import config_083
from .config_084 import config_084
from .config_085 import config_085
from .config_086 import config_086
from .config_087 import config_087
from .config_088 import config_088
from .config_089 import config_089
from .config_090 import config_090
from .config_091 import config_091
from .config_092 import config_092
from .config_093 import config_093
from .config_094 import config_094
from .config_095 import config_095
from .config_096 import config_096
from .config_097 import config_097
from .config_098 import config_098
from .config_099 import config_099
from .config_100 import config_100
from .config_101 import config_101
from .config_102 import config_102
from .config_103 import config_103
from .config_104 import config_104
from .config_105 import config_105
from .config_106 import config_106
from .config_107 import config_107
from .config_108 import config_108
from .config_109 import config_109
from .config_110 import config_110

# from .config_111 import config_111
# from .config_112 import config_112
# from .config_113 import config_113
# from .config_114 import config_114
# from .config_115 import config_115
# from .config_116 import config_116
# from .config_117 import config_117
# from .config_118 import config_118
# from .config_119 import config_119
# from .config_120 import config_120
# from .config_121 import config_121
# from .config_122 import config_122
# from .config_123 import config_123
# from .config_124 import config_124
# from .config_125 import config_125
# from .config_126 import config_126
from .config_127 import config_127
from .config_128 import config_128
from .config_129 import config_129
from .config_130 import config_130

# from .config_131 import config_131
# from .config_132 import config_132
# from .config_133 import config_133
# from .config_134 import config_134
from .config_135 import config_135
from .config_136 import config_136
from .config_137 import config_137
from .config_138 import config_138

# from .config_139 import config_139
# from .config_140 import config_140
# from .config_141 import config_141
# from .config_142 import config_142
# from .config_143 import config_143
# from .config_144 import config_144
# from .config_145 import config_145
# from .config_146 import config_146
# from .config_147 import config_147
# from .config_148 import config_148
# from .config_149 import config_149
# from .config_150 import config_150
# from .config_151 import config_151
# from .config_152 import config_152
# from .config_153 import config_153
# from .config_154 import config_154
# from .config_155 import config_155
# from .config_156 import config_156
# from .config_157 import config_157
# from .config_158 import config_158
from .config_159 import config_159
from .config_160 import config_160
from .config_161 import config_161
from .config_162 import config_162
from .config_163 import config_163
from .config_164 import config_164
from .config_165 import config_165
from .config_166 import config_166
from .config_167 import config_167
from .config_168 import config_168
from .config_169 import config_169
from .config_170 import config_170
from .config_171 import config_171
from .config_172 import config_172
from .config_173 import config_173
from .config_174 import config_174
from .config_175 import config_175
from .config_176 import config_176
from .config_177 import config_177
from .config_178 import config_178
from .config_179 import config_179
from .config_180 import config_180
from .config_181 import config_181
from .config_182 import config_182
from .config_183 import config_183
from .config_184 import config_184
from .config_185 import config_185
from .config_186 import config_186
from .config_187 import config_187
from .config_188 import config_188
from .config_189 import config_189
from .config_190 import config_190
from .config_191 import config_191
from .config_192 import config_192
from .config_193 import config_193
from .config_194 import config_194
from .config_195 import config_195
from .config_196 import config_196
from .config_197 import config_197
from .config_198 import config_198
from .config_199 import config_199
from .config_200 import config_200
from .config_201 import config_201
from .config_202 import config_202
from .config_203 import config_203
from .config_204 import config_204
from .config_205 import config_205
from .config_206 import config_206
from .config_207 import config_207
from .config_208 import config_208
from .config_209 import config_209
from .config_210 import config_210
from .config_211 import config_211
from .config_212 import config_212
from .config_213 import config_213
from .config_214 import config_214
from .config_215 import config_215
from .config_216 import config_216
from .config_217 import config_217
from .config_218 import config_218
from .config_219 import config_219
from .config_220 import config_220


__all__ = [
    "config_035",
    "config_036",
    "config_037",
    "config_038",
    "config_039",
    "config_040",
    "config_041",
    "config_042",
    "config_043",
    "config_044",
    "config_045",
    "config_046",
    "config_047",
    "config_048",
    "config_049",
    "config_050",
    "config_051",
    "config_052",
    "config_053",
    "config_054",
    "config_055",
    "config_056",
    "config_057",
    "config_058",
    "config_059",
    "config_060",
    "config_061",
    "config_062",
    "config_063",
    "config_064",
    "config_065",
    "config_066",
    "config_067",
    "config_068",
    "config_069",
    "config_070",
    "config_071",
    "config_072",
    "config_073",
    "config_074",
    "config_075",
    "config_076",
    "config_077",
    "config_078",
    "config_079",
    "config_080",
    "config_081",
    "config_082",
    "config_083",
    "config_084",
    "config_085",
    "config_086",
    "config_087",
    "config_088",
    "config_089",
    "config_090",
    "config_091",
    "config_092",
    "config_093",
    "config_094",
    "config_095",
    "config_096",
    "config_097",
    "config_098",
    "config_099",
    "config_100",
    "config_101",
    "config_102",
    "config_103",
    "config_104",
    "config_105",
    "config_106",
    "config_107",
    "config_108",
    "config_109",
    "config_110",
    # "config_111",
    # "config_112",
    # "config_113",
    # "config_114",
    # "config_115",
    # "config_116",
    # "config_117",
    # "config_118",
    # "config_119",
    # "config_120",
    # "config_121",
    # "config_122",
    # "config_123",
    # "config_124",
    # "config_125",
    # "config_126",
    "config_127",
    "config_128",
    "config_129",
    "config_130",
    # "config_131",
    # "config_132",
    # "config_133",
    # "config_134",
    "config_135",
    "config_136",
    "config_137",
    "config_138",
    # "config_139",
    # "config_140",
    # "config_141",
    # "config_142",
    # "config_143",
    # "config_144",
    # "config_145",
    # "config_146",
    # "config_147",
    # "config_148",
    # "config_149",
    # "config_150",
    # "config_151",
    # "config_152",
    # "config_153",
    # "config_154",
    # "config_155",
    # "config_156",
    # "config_157",
    # "config_158",
    "config_159",
    "config_160",
    "config_161",
    "config_162",
    "config_163",
    "config_164",
    "config_165",
    "config_166",
    "config_167",
    "config_168",
    "config_169",
    "config_170",
    "config_171",
    "config_172",
    "config_173",
    "config_174",
    "config_175",
    "config_176",
    "config_177",
    "config_178",
    "config_179",
    "config_180",
    "config_181",
    "config_182",
    "config_183",
    "config_184",
    "config_185",
    "config_186",
    "config_187",
    "config_188",
    "config_189",
    "config_190",
    "config_191",
    "config_192",
    "config_193",
    "config_194",
    "config_195",
    "config_196",
    "config_197",
    "config_198",
    "config_199",
    "config_200",
    "config_201",
    "config_202",
    "config_203",
    "config_204",
    "config_205",
    "config_206",
    "config_207",
    "config_208",
    "config_209",
    "config_210",
    "config_211",
    "config_212",
    "config_213",
    "config_214",
    "config_215",
    "config_216",
    "config_217",
    "config_218",
    "config_219",
    "config_220",
]

engine_test_configs = [
    config_035,
    config_036,
    config_037,
    config_038,
    config_039,
    config_040,
    config_041,
    config_042,
    config_043,
    config_044,
    config_045,
    config_046,
    config_047,
    config_048,
    config_049,
    config_050,
    config_051,
    config_052,
    config_053,
    config_054,
    config_055,
    config_056,
    config_057,
    config_058,
    config_059,
    config_060,
    config_061,
    config_062,
    config_063,
    config_064,
    config_065,
    config_066,
    config_067,
    config_068,
    config_069,
    config_070,
    config_071,
    config_072,
    config_073,
    config_074,
    config_075,
    config_076,
    config_077,
    config_078,
    config_079,
    config_080,
    config_081,
    config_082,
    config_083,
    config_084,
    config_085,
    config_086,
    config_087,
    config_088,
    config_089,
    config_090,
    config_091,
    config_092,
    config_093,
    config_094,
    config_095,
    config_096,
    config_097,
    config_098,
    config_099,
    config_100,
    config_101,
    config_102,
    config_103,
    config_104,
    config_105,
    config_106,
    config_107,
    config_108,
    config_109,
    config_110,
    # config_111,
    # config_112,
    # config_113,
    # config_114,
    # config_115,
    # config_116,
    # config_117,
    # config_118,
    # config_119,
    # config_120,
    # config_121,
    # config_122,
    # config_123,
    # config_124,
    # config_125,
    # config_126,
    config_127,
    config_128,
    config_129,
    config_130,
    # config_131,
    # config_132,
    # config_133,
    # config_134,
    config_135,
    config_136,
    config_137,
    config_138,
    # config_139,
    # config_140,
    # config_141,
    # config_142,
    # config_143,
    # config_144,
    # config_145,
    # config_146,
    # config_147,
    # config_148,
    # config_149,
    # config_150,
    # config_151,
    # config_152,
    # config_153,
    # config_154,
    # config_155,
    # config_156,
    # config_157,
    # config_158,
    config_159,
    config_160,
    config_161,
    config_162,
    config_163,
    config_164,
    config_165,
    config_166,
    config_167,
    config_168,
    config_169,
    config_170,
    config_171,
    config_172,
    config_173,
    config_174,
    config_175,
    config_176,
    config_177,
    config_178,
    config_179,
    config_180,
    config_181,
    config_182,
    config_183,
    config_184,
    config_185,
    config_186,
    config_187,
    config_188,
    config_189,
    config_190,
    config_191,
    config_192,
    config_193,
    config_194,
    config_195,
    config_196,
    config_197,
    config_198,
    config_199,
    config_200,
    config_201,
    config_202,
    config_203,
    config_204,
    config_205,
    config_206,
    config_207,
    config_208,
    config_209,
    config_210,
    config_211,
    config_212,
    config_213,
    config_214,
    config_215,
    config_216,
    config_217,
    config_218,
    config_219,
    config_220,
]



from frozendict import frozendict

from bgpy.tests.engine_tests.graphs import graph_008
from bgpy.tests.engine_tests.utils import EngineTestConfig

from bgpy.simulation_engine import BGPSimpleAS
from bgpy.enums import ASNs
from bgpy.simulation_framework import ScenarioConfig, SubprefixHijack

from rovpp import ROVPPV2SimpleAS, ROVPPAnn


config_047 = EngineTestConfig(
    name="047",
    desc=(
        "Subprefix Hijack that tests blackhole announcements aren't sent"
        " if hijack comes from customers with ROV++ v2"
    ),
    scenario_config=ScenarioConfig(
        ScenarioCls=SubprefixHijack,
        BaseASCls=BGPSimpleAS,
        AdoptASCls=ROVPPV2SimpleAS,
        override_attacker_asns=frozenset({ASNs.ATTACKER.value}),
        override_victim_asns=frozenset({ASNs.VICTIM.value}),
        override_non_default_asn_cls_dict=frozendict({1: ROVPPV2SimpleAS}),
        AnnCls=ROVPPAnn,
    ),
    graph=graph_008,
)



from frozendict import frozendict

from bgpy.tests.engine_tests.graphs import graph_010
from bgpy.tests.engine_tests.utils import EngineTestConfig

from bgpy.simulation_engine import BGPSimpleAS, ROVSimpleAS
from bgpy.enums import ASNs
from bgpy.simulation_framework import ScenarioConfig, SuperprefixPrefixHijack

from rovpp import ROVPPAnn


config_065 = EngineTestConfig(
    name="065",
    desc="ROV adopting AS 2. It is not useful in this type of attack",
    scenario_config=ScenarioConfig(
        ScenarioCls=SuperprefixPrefixHijack,
        BaseASCls=BGPSimpleAS,
        AdoptASCls=ROVSimpleAS,
        override_attacker_asns=frozenset({ASNs.ATTACKER.value}),
        override_victim_asns=frozenset({ASNs.VICTIM.value}),
        override_non_default_asn_cls_dict=frozendict({2: ROVSimpleAS}),
        AnnCls=ROVPPAnn,
    ),
    graph=graph_010,
)



from frozendict import frozendict

from bgpy.tests.engine_tests.graphs import graph_003
from bgpy.tests.engine_tests.utils import EngineTestConfig
from bgpy.simulation_engine import BGPSimpleAS
from bgpy.enums import ASNs
from bgpy.simulation_framework import ScenarioConfig, SubprefixHijack

from rovpp import ROVPPV1SimpleAS, ROVPPAnn

config_037 = EngineTestConfig(
    name="rovpp_037",
    desc="Subprefix Hijack from fig 2 in paper with ROV++ v1 adopting.",
    scenario_config=ScenarioConfig(
        ScenarioCls=SubprefixHijack,
        BaseASCls=BGPSimpleAS,
        AdoptASCls=ROVPPV1SimpleAS,
        AnnCls=ROVPPAnn,
        override_attacker_asns=frozenset({ASNs.ATTACKER.value}),
        override_victim_asns=frozenset({ASNs.VICTIM.value}),
        override_non_default_asn_cls_dict=frozendict(
            {3: ROVPPV1SimpleAS, 4: ROVPPV1SimpleAS}
        ),
    ),
    graph=graph_003,
)



from .v2_aggressive import ROVPPV2aLiteSimpleAS
from .v2_aggressive import ROVPPV2aSimpleAS
from .v2_base import ROVPPV2LiteSimpleAS
from .v2_base import ROVPPV2SimpleAS
from .v2_base_shorten import ROVPPV2ShortenLiteSimpleAS
from .v2_base_shorten import ROVPPV2ShortenSimpleAS
from .v2_journal import ROVPPV2JournalLiteSimpleAS
from .v2_journal import ROVPPV2JournalSimpleAS


__all__ = [
    "ROVPPV2aLiteSimpleAS",
    "ROVPPV2aSimpleAS",
    "ROVPPV2LiteSimpleAS",
    "ROVPPV2SimpleAS",
    "ROVPPV2ShortenLiteSimpleAS",
    "ROVPPV2ShortenSimpleAS",
    "ROVPPV2JournalLiteSimpleAS",
    "ROVPPV2JournalSimpleAS",
]



from .v1 import ROVPPV1LiteSimpleAS
from .v1 import ROVPPV1SimpleAS
from .v2 import ROVPPV2LiteSimpleAS
from .v2 import ROVPPV2SimpleAS
from .v2 import ROVPPV2aLiteSimpleAS
from .v2 import ROVPPV2aSimpleAS
from .v2 import ROVPPV2ShortenSimpleAS
from .v2 import ROVPPV2ShortenLiteSimpleAS
from .v2 import ROVPPV2JournalSimpleAS
from .v2 import ROVPPV2JournalLiteSimpleAS
from .rovpp_v3_as import ROVPPV3AS

__all__ = [
    "ROVPPV1LiteSimpleAS",
    "ROVPPV2LiteSimpleAS",
    "ROVPPV2aLiteSimpleAS",
    "ROVPPV1SimpleAS",
    "ROVPPV2SimpleAS",
    "ROVPPV2aSimpleAS",
    "ROVPPV2ShortenSimpleAS",
    "ROVPPV2ShortenLiteSimpleAS",
    "ROVPPV2JournalSimpleAS",
    "ROVPPV2JournalLiteSimpleAS",
    "ROVPPV3AS",
]



from pathlib import Path
import subprocess

import pytest

from bgpy.caida_collector import CaidaCollector
from bgpy import DiagramAggregator


DIAGRAM_PATH = Path(__file__).parent / "engine_tests" / "engine_test_outputs"


# https://github.com/pytest-dev/pytest-xdist/issues/783
def pytest_configure(config):
    """Caches the caida collector before parallelization"""

    # Prevent workers from running the same code
    if not hasattr(config, "workerinput"):
        # Caches CAIDA downloaded file only once before tests run
        CaidaCollector().run(tsv_path=None)


def pytest_sessionfinish(session, exitstatus):
    """Runs when all tests are done, once per session, even with xdist

    https://github.com/pytest-dev/pytest-xdist/issues/271#issuecomment-826396320  # noqa
    """

    # Only run in master thread after all other threads/tests have finished
    # Also runs when xdist isn't running
    if not hasattr(session.config, "workerinput"):
        DiagramAggregator(DIAGRAM_PATH).aggregate_diagrams()
        # Teardown stuff (open PDF for viewing)
        if session.config.getoption("view"):
            # https://stackoverflow.com/q/19453338/8903959
            agg_path = DiagramAggregator(DIAGRAM_PATH).aggregated_diagrams_path
            subprocess.call(["xdg-open", str(agg_path)])


@pytest.fixture(scope="session")
def overwrite(pytestconfig):
    return pytestconfig.getoption("overwrite")


# https://stackoverflow.com/a/66597438/8903959
def pytest_addoption(parser):
    parser.addoption("--view", action="store_true", default=False)
    parser.addoption("--overwrite", action="store_true", default=False)
